package uniandes.cupi2.sitp.test;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import estructuras.listas.ListaDoble;
import junit.framework.TestCase;
import uniandes.cupi2.sitp.mundo.Dia;
import uniandes.cupi2.sitp.mundo.Parada;
import uniandes.cupi2.sitp.mundo.Ruta;

public abstract class RutaTest extends TestCase 
{
	protected Ruta ruta;
	
	protected abstract void setUp();
	
	protected void agregarParadas()
	{
		Parada p1 = new Parada("p1", 10, 10);
		Parada p2 = new Parada("p2", 20, 10);
		Parada p3 = new Parada("p3", 30, 10);
		Parada p4 = new Parada("p4", 40, 10);
		Parada p5 = new Parada("p5", 50, 10);
		try {
			ruta.agregarParada(p1);
			ruta.agregarParada(p2);
			ruta.agregarParada(p3);
			ruta.agregarParada(p4);
			ruta.agregarParada(p5);
			
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
			fail("No debera generar excepciones");
		}
	}
	
	protected void asignarNombre()
	{
		ruta.cambiarNombre("ruta");
	}
	
	protected void asignarHoras()
	{
		Calendar c = Calendar.getInstance();
		c.set(Calendar.HOUR_OF_DAY, 4);
		c.set(Calendar.MINUTE, 30);
		Date inicio = c.getTime();
		
		c.set(Calendar.HOUR_OF_DAY, 22);
		c.set(Calendar.MINUTE, 00);
		Date fin = c.getTime();
		
		ruta.cambiarHoraInicio(inicio);
		ruta.cambiarHoraFin(fin);
	}
	
	protected void asignarDias()
	{
		List<Dia> dias = new ArrayList<Dia>();
		dias.add(Dia.LUNES);
		dias.add(Dia.MARTES);
		dias.add(Dia.MIERCOLES);
		dias.add(Dia.JUEVES);
		dias.add(Dia.VIERNES);
		ruta.cambiarDias(dias);
	}
	
	public void testDarParadas()
	{
		assertTrue("La lsita debe ser doblemente encadenada", ruta.darParadas() instanceof ListaDoble<?>);
		assertEquals("Debera haber 5 paradas", 5, ruta.darParadas().size());
		for (int i = 1; i <= 5; i++) 
		{
			assertEquals("La parada debe tener id p"+i, "p"+i, ruta.darParadas().get(i-1).darNombre());
		}
	}
	
	public void testAgregarParadaOK()
	{
		Parada p = new Parada("p6", 50, 20);
		try 
		{
			ruta.agregarParada(p);
			assertEquals("Deberan existir 6 paradas", 6, ruta.darParadas().size());
			assertEquals("La parada se debe agregar al final de la lista", "p6", ruta.darParadas().get(ruta.darParadas().size()-1).darNombre());
		} 
		catch (Exception e) 
		{
			fail("No debe generar errores");
		}
	}
	
	public void testAgregarParadaError()
	{
		Parada p = new Parada("p2", 50, 20);
		try 
		{
			ruta.agregarParada(p);
			fail("No debe agregar la parada");
		} 
		catch (Exception e) 
		{
			assertEquals("Deberan existir 5 paradas", 5, ruta.darParadas().size());
			for (int i = 1; i <= 5; i++) 
			{
				assertEquals("La parada debe tener id p"+i, "p"+i, ruta.darParadas().get(i-1).darNombre());
			}
		}
	}
	
	public void testDarDias()
	{
		List<Dia> ds = ruta.darDiasFuncionamiento();
		assertTrue("Debera operar el lunes", ds.contains(Dia.LUNES));
		assertTrue("Debera operar el martes", ds.contains(Dia.MARTES));
		assertTrue("Debera operar el mircoles", ds.contains(Dia.MIERCOLES));
		assertTrue("Debera operar el jueves", ds.contains(Dia.JUEVES));
		assertTrue("Debera operar el viernes", ds.contains(Dia.VIERNES));
	}
	
	public void testDarNombre()
	{
		assertEquals("El nombre debe ser ruta", "ruta", ruta.darNombre());
	}
	
	public void testDarHoras()
	{
		Calendar c = Calendar.getInstance();
		c.setTime(ruta.darHoraInicioFuncionamiento());
		assertTrue("La ruta debera inciar a las 4:30", c.get(Calendar.HOUR_OF_DAY) == 4 && c.get(Calendar.MINUTE) == 30);
		
		c.setTime(ruta.darHoraFinFuncionamiento());
		assertTrue("La ruta debera terminar a las 22:00", c.get(Calendar.HOUR_OF_DAY) == 22 && c.get(Calendar.MINUTE) == 0);
	}
	
	

}
