package uniandes.cupi2.sitp.test;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import estructuras.listas.ListaSencilla;
import junit.framework.TestCase;
import uniandes.cupi2.sitp.mundo.Alimentador;
import uniandes.cupi2.sitp.mundo.Articulado;
import uniandes.cupi2.sitp.mundo.BusUrbano;
import uniandes.cupi2.sitp.mundo.Dia;
import uniandes.cupi2.sitp.mundo.FabricaRutas;
import uniandes.cupi2.sitp.mundo.IRuta;
import uniandes.cupi2.sitp.mundo.Parada;
import uniandes.cupi2.sitp.mundo.SITP;
import uniandes.cupi2.sitp.mundo.Troncal;

public class SITPTest extends TestCase
{
	private SITP sitp;
	
	public void setUp()
	{
		sitp = new SITP();
		
		List<Dia> semana = new ArrayList<Dia>();
		semana.add(Dia.LUNES);
		semana.add(Dia.MARTES);
		semana.add(Dia.MIERCOLES);
		semana.add(Dia.JUEVES);
		semana.add(Dia.VIERNES);
		
		Calendar c = Calendar.getInstance();
		c.set(Calendar.HOUR_OF_DAY, 4);
		c.set(Calendar.MINUTE, 30);
		Date inicio = c.getTime();
		
		c.set(Calendar.HOUR_OF_DAY, 22);
		c.set(Calendar.MINUTE, 00);
		Date fin = c.getTime();
		
		ArrayList<Troncal> ts = new ArrayList<Troncal>();
		ts.add(Troncal.A);
		ts.add(Troncal.B);
		ts.add(Troncal.C);
		
		Parada p1 = new Parada("p1", 10, 10);
		Parada p2 = new Parada("p2", 20, 10);
		Parada p3 = new Parada("p3", 30, 10);
		Parada p4 = new Parada("p4", 40, 10);
		
		IRuta r1 = FabricaRutas.crearAlimentador("Alimentador1", semana, p1, inicio, fin);
		IRuta r2 = FabricaRutas.crearBusUrbano("BusUrbano1", semana, p2, inicio, fin);
		IRuta r3 = FabricaRutas.crearArticulado("Articulado1", semana, p3, inicio, fin, ts);
		IRuta r4 = FabricaRutas.crearBusUrbano("BusUrbano2", semana, p4, inicio, fin);
		
		try 
		{
			sitp.agregarRuta(r1);
			sitp.agregarRuta(r2);
			sitp.agregarRuta(r3);
			sitp.agregarRuta(r4);
		} 
		catch (Exception e) {
			fail("No debera ocurrir excepcin");
		}
		
	}
	
	public void testDarRutas()
	{
		List<IRuta> rs = sitp.darRutas();
		assertNotNull("Debe devolver una lista", rs);
		assertTrue("La lista debe ser una lista sencilla", rs instanceof ListaSencilla<?>);
		assertTrue("La lista debe tener 4 elementos", rs.size() == 4);
		assertTrue("El primer elemento debe ser un alimentdor", rs.get(0) instanceof Alimentador);
		assertTrue("El segundo elemento debe ser un bus urbano", rs.get(1) instanceof BusUrbano);
		assertTrue("El tercer elemento debe ser un articulado", rs.get(2) instanceof Articulado);
		assertTrue("El cuarto elemento debe ser un bus urbano", rs.get(3) instanceof BusUrbano);
	}
	
	public void testAgregarRutaOk()
	{
		List<Dia> finSemana = new ArrayList<Dia>();
		finSemana.add(Dia.SABADO);
		finSemana.add(Dia.DOMINGO);
		
		Calendar c = Calendar.getInstance();
		c.set(Calendar.HOUR_OF_DAY, 5);
		c.set(Calendar.MINUTE, 30);
		Date inicio = c.getTime();
		
		c.set(Calendar.HOUR_OF_DAY, 15);
		c.set(Calendar.MINUTE, 00);
		Date fin = c.getTime();

		
		Parada p = new Parada("p1", 50, 40);
		IRuta r = FabricaRutas.crearAlimentador("Alimentador2", finSemana, p, inicio, fin);
		try 
		{
			sitp.agregarRuta(r);
			List<IRuta> rs = sitp.darRutas();
			assertTrue("La lista debe tener 5 elementos", rs.size() == 5);
			assertTrue("El quinto elemento debe ser un alimentdor", rs.get(4) instanceof Alimentador);
			
		} 
		catch (Exception e) 
		{
			fail("No debera generar errores");
		}
		
	}
	
	public void testAgregarRutaError()
	{
		List<Dia> finSemana = new ArrayList<Dia>();
		finSemana.add(Dia.SABADO);
		finSemana.add(Dia.DOMINGO);
		
		Calendar c = Calendar.getInstance();
		c.set(Calendar.HOUR_OF_DAY, 5);
		c.set(Calendar.MINUTE, 30);
		Date inicio = c.getTime();
		
		c.set(Calendar.HOUR_OF_DAY, 15);
		c.set(Calendar.MINUTE, 00);
		Date fin = c.getTime();

		
		Parada p = new Parada("p1", 50, 40);
		IRuta r = FabricaRutas.crearAlimentador("Alimentador1", finSemana, p, inicio, fin);
		try 
		{
			sitp.agregarRuta(r);
			fail("Debera generar una excepcin");
			
		} 
		catch (Exception e) 
		{
			List<IRuta> rs = sitp.darRutas();
			assertTrue("La lista debe tener 4 elementos", rs.size() == 4);
		}
	}
	
	public void testEliminarPrimero()
	{
		IRuta r = sitp.darRutas().get(0);
		sitp.eliminarRuta(r);
		List<IRuta> rs = sitp.darRutas();
		
		assertTrue("La lista debe tener 3 elementos", rs.size() == 3);
		assertTrue("El primer elemento debe ser un bus urbano", rs.get(0) instanceof BusUrbano);
		assertTrue("El segundo elemento debe ser un articulado", rs.get(1) instanceof Articulado);
		assertTrue("El tercer elemento debe ser un bus urbano", rs.get(2) instanceof BusUrbano);
	}
	
	public void testEliminarMedio()
	{
		IRuta r = sitp.darRutas().get(1);
		sitp.eliminarRuta(r);
		List<IRuta> rs = sitp.darRutas();
		
		assertTrue("La lista debe tener 3 elementos", rs.size() == 3);
		assertTrue("El primer elemento debe ser unAlimentador", rs.get(0) instanceof Alimentador);
		assertTrue("El segundo elemento debe ser un articulado", rs.get(1) instanceof Articulado);
		assertTrue("El tercer elemento debe ser un bus urbano", rs.get(2) instanceof BusUrbano);
	}
	
	public void testEliminarUltimo()
	{
		IRuta r = sitp.darRutas().get(3);
		sitp.eliminarRuta(r);
		List<IRuta> rs = sitp.darRutas();
		
		assertTrue("La lista debe tener 3 elementos", rs.size() == 3);
		assertTrue("El primer elemento debe ser un alimentdor", rs.get(0) instanceof Alimentador);
		assertTrue("El segundo elemento debe ser un bus urbano", rs.get(1) instanceof BusUrbano);
		assertTrue("El tercer elemento debe ser un articulado", rs.get(2) instanceof Articulado);
	}
	
	public void testDarRutaCercanaExiste()
	{
		IRuta r = sitp.darRutaCercana(11, 11);
		assertNotNull("Debera encontrar ruta", r);
		assertEquals("La ruta encontrada debera ser la Alimentador1", "Alimentador1", r.darNombre());
	}
	
	
	public void testDarRutaCercanaNoExiste()
	{
		IRuta r = sitp.darRutaCercana(130, 150);
		assertNull("No debera encontrar ruta", r);
	}
	
	public void testBuscarRutaNombreExiste()
	{
		IRuta r = sitp.buscarRutaNombre("bano1");

		assertNotNull("Debera encontrar ruta", r);
		assertEquals("La ruta encontrada debera ser la BusUrbano1", "BusUrbano1", r.darNombre());
	}
	
	public void  testBuscarRutaNombreNoExiste()
	{
		IRuta r = sitp.buscarRutaNombre("santafe");
		assertNull("No debera encontrar ruta", r);
	}
}
