package estructuras.listas;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Iterador simple sobre una lista sencillamente encadenada
 * @author alvar-go
 * @param <T> tipo de elementos que almacena la lista sobre la que se itera
 */
public class IteradorSencillamenteEncadenada<T  extends IUnicamenteIdentificado> implements Iterator<T> {

	/**
	 * Nodo en el que se encuentra actualmente el iterador
	 */
	private NodoListaSencilla<T> actual;
	
	/**
	 * Crea un nuevo iterador
	 * post:	el elemento actual se ha inicializado en el elemento por parmetro
	 * @param nActual el nodo en el que inicia el iterador.
	 */
	public IteradorSencillamenteEncadenada(NodoListaSencilla<T> nActual)
	{
		actual = nActual;
	}
	

	/**
	 * Indica si an hay elementos por iterar
	 * @return true en caso que an queden elementos por iterar o false en caso contrario
	 */
	public boolean hasNext() 
	{
		return actual != null;
	}

	
	/**
	 * Devuelve el elemento del nodo actual
	 * post:	Se ha avanzado el nodo actual al siguiente del actual
	 * @return 	elemento del nodo actual
	 * @throws NoSuchElementException Se lanza en caso que la lista no tenga ms elementos
	 */
	public T next() throws NoSuchElementException
	{
		if( actual == null )
		{
			throw new NoSuchElementException("No hay ms elementos en la lista");
		}
		T elem = actual.darElemento();
		actual = actual.darSiguiente();
		return elem;
	}

}
