package estructuras.listas;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

/**
 * Clase que representa un lista sencillamente encadenada. La lista no acepta elementos nulos o repetidos
 * @author alvar-go
 * @param <T> tipo de elementos almacenados en la lista
 */
public class ListaDoble<T  extends IUnicamenteIdentificado> implements List<T>, Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 70L;
	/**
	 * Primer elemento de la lista
	 */

	//TODO declare el atributo que almacena el primer nodo de la lista, llamelo primero
	
	/**
	 * Crea una lista sencillamente encadenada
	 * post:	El primer elemento se ha inicializado en null
	 */
	public ListaDoble()
	{

		//TODO complete segn la documentacin
	}
	
	/**
	 * Agrega el elemento por parmetro al final de la lista
	 * @param t el elemento que se desea agregar.
	 * @return true en caso de que la lista se haya modificado (se agreg el elemento) o false en caso contrario
	 * @throws NullPointerException Se lanza en caso que T sea nulo
	 * @throws IllegalArgumentException Se lanza en caso que ya exista un elemento con el identificador de t en la lista
	 */
	public boolean add(T t) throws NullPointerException, IllegalArgumentException
	{
		//TODO complete segn la documentacin
		//Revise las excepciones
		//Revise el caso en que la lista est vaca
		//Si la lista tiene elementos, avance hata el final de la lista y agregue el elemento en un nuevo nodo
	
	}

	/**
	 * Agrega el elemento t en la posicin pos de la lista
	 * post:  Los elementos posteriores a pos se han movido una posicin a la derecha
	 * @param pos la posicin donde se desea agregar elemento
	 * @param t el elemento que se desea agregar
	 * @throws NullPointerException En caso que t sea nulo
	 * @throws IndexOutOfBoundsException si la posicin est fuera de la lista (pos < 0 || pos > size())
	 * @throws IllegalArgumentException si ya existe el elemento t en la lista
	 */
	public void add(int pos, T t) throws NullPointerException, IndexOutOfBoundsException, IllegalArgumentException
	{
		//TODO complete segn la documentacin
		
		
	}


	/**
	 * Agrega los elementos de la coleccin c al final de la lista en el orden que los devuelva el iterador de c
	 * @param la coleccin de elementos que se desea agregar
	 * @return true en caso de que se agregue al menos un elemento a la lista o false en caso contrario.
	 * @throws NullPointerException Se lanza en caso que algn t en c sea nulo. En este caso los elementos previos a t se agregan y se detiene la operacin
	 * al encontrar un elemento nulo.
	 * @throws IllegalArgumentException Se lanza en caso que ya exista un elemento t de c en la lista.En este caso los elementos previos a t se agregan y se detiene la operacin
	 * al encontrar un elemento repetido.
	 */
	public boolean addAll(Collection<? extends T> c) throws NullPointerException, IllegalArgumentException
	{
		boolean modificado = false;
		Iterator<? extends T> it = c.iterator();
		while(it.hasNext())
		{
			modificado |= add(it.next());
		}
		return modificado;
	}


	/**
	 * Agrega los elementos de la coleccin c de la lista en el orden que los devuelva el iterador de c iniciando en la posicin que llega por parmetro 
	 * post:  Los elementos posteriores a pos se han movido pos posiciones a la derecha
	 * @param la coleccin de elementos que se desea agregar
	 * @param la posicin a partir de la cual se quieren agregar los elementos en la lista
	 * @return true en caso de que se agregue al menos un elemento a la lista o false en caso contrario.
	 * @throws NullPointerException Se lanza en caso que algn t en c sea nulo. En este caso los elementos previos a t se agregan y se detiene la operacin
	 * al encontrar un elemento nulo.
	 * @throws IllegalArgumentException Se lanza en caso que ya exista un elemento t de c en la lista.En este caso los elementos previos a t se agregan y se detiene la operacin
	 * al encontrar un elemento repetido.
	 * @throws IndexOutOfBoundsException si la posicin est fuera de la lista (pos < 0 || pos > size())
	 */
	public boolean addAll(int pos, Collection<? extends T> c) throws NullPointerException, IndexOutOfBoundsException, IllegalArgumentException
	{
		if(pos < 0 || pos > size())
		{
			throw new IndexOutOfBoundsException();
		}
		boolean modificado = false;
		Iterator<? extends T> it = c.iterator();
		int i = 0;
		while(it.hasNext())
		{
			add(pos+i,it.next());
			modificado = true;
			i++;
		}
		return modificado;
	}


	/**
	 * Eliminad todos los elementos de la lista
	 * post: se han borrado todos los elementos de la lista
	 */
	public void clear() 
	{
		//TODO complete segn la documentacin
		
	}

	/**
	 * Indica si la lista contiene el objeto que llega por parmetro
	 * @param o el objeto que se desea buscar en la lista
	 * @return true si la lista contiene el elemento o false en caso contrario
	 * @throws ClassCastException En caso de que o no sea de tipo T
	 * @throws NullPointerException En caso que o sea nulo
	 */
	public boolean contains(Object o) throws NullPointerException, ClassCastException
	{
		//TODO complete segn la documentacin
		// Revise que o no sea nulo
		// Revise que o sea instancia de IUnicamenteIdentificado
		// Recorrra la lista buscando el elemento
		
	}
	
	

	/**
	 * Indica si la lista contiene todos los objetos de la coleccin c
	 * @param c la coelccin de objetos que se quiere buscar en la lista
	 * @return true si la lista contiene todos los elementos de c o false en caso contrario
	 * @throws ClassCastException En caso de que algn elemento de c no sea de tipo T
	 * @throws NullPointerException En caso que  algn elemento de c sea nulo
	 */
	public boolean containsAll(Collection<?> c) throws NullPointerException, ClassCastException
	{
		Iterator<?> it = c.iterator();
		boolean contiene = true;
		while(it.hasNext() && contiene)
		{
			contiene &= contains(it.next());
		}
		return contiene;
	}

	/**
	 * Devuelve el elemento en la posicin dada por parmetro
	 * @param pos la posicin de la que se desea conocer el elemento
	 * @throws IndexOutOfBoundsException si la posicin est fuera de la lista (pos < 0 || pos > size())
	 */
	public T get(int pos) 
	{
		//TODO compelte segn la documentacin
		
	}

	/**
	 * Devuelve el indice de la primera aparicin de o
	 * @param o el objeto del que se desea conocer la posicin en la lista
	 * @return la posicin del objeto en la lista o -1 en caso de que no se encuentre en ella
	 * @throws ClassCastException En caso de o no sea de tipo T
	 * @throws NullPointerException En caso que o sea nulo
	 */
	public int indexOf(Object o) 
	{
		if(o==null)
		{
			throw new NullPointerException("La lista no acepta elementos nulos");
		}
		if( o instanceof IUnicamenteIdentificado)
		{
			int  pos = -1;
			IUnicamenteIdentificado obj = (IUnicamenteIdentificado)o;
			pos = 0;
			NodoListaDoble<T> actual = primero;
			while(actual != null && !actual.darElemento().darIdentificador().equals(obj.darIdentificador()))
			{
				actual = actual.darSiguiente();
				pos++;
			}
			
			return pos;
		}
		else
		{
			throw new ClassCastException("La lista no acepta elementos: " +o.getClass().getName());
		}
	}

	/**
	 * Indica si la lista esta vaca
	 * @return true en caso que la lista no tenga elementos o false en caso contrario
	 */
	public boolean isEmpty() 
	{
		return primero == null;
	}

	/**
	 * Devuelve un iterador sobre la lista
	 * @return un nuevo iterador que inicia al principio de la lista
	 */
	public Iterator<T> iterator() 
	{
		return new IteradorDoblementeEncadenada<>(primero);
	}

	/**
	 * Devuelve el indice de la ltima aparicin de o. Considerando que la lista no acepta elementos repetidos es equivalente a indexOf
	 * @param o el objeto del que se desea conocer la posicin en la lista
	 * @return la posicin del objeto en la lista o -1 en caso de que no se encuentre en ella
	 * @throws ClassCastException En caso de o no sea de tipo T
	 * @throws NullPointerException En caso que o sea nulo
	 */
	public int lastIndexOf(Object o) 
	{
		return indexOf(o);
	}

	/**
	 * Devuelve un iterador de lista sobre la lista
	 * @return nuevo iterador que inicia al principio de la lista
	 */
	public ListIterator<T> listIterator() 
	{
		return new IteradorListaDoblementeEncadenada<>(primero);
	}

	
	/**
	 * Devuelve un iterador de lista sobre la lista
	 * @return nuevo iterador que inicia en la posicin indicada de la lista
	 */
	public ListIterator<T> listIterator(int pos) 
	{
		return new IteradorListaDoblementeEncadenada<>(primero, pos);
	}

	/**
	 * Elimina el objeto o de la lista en caso que exista en ella
	 * @param o le objeto que se desea eliminar
	 * @return true si el elemento existia y se elimino o false en caso contrario
	 * @throws ClassCastException En caso de o no sea de tipo T
	 * @throws NullPointerException En caso que o sea nulo
	 */
	public boolean remove(Object o) 
	{
		boolean eliminado = false;
		int i = indexOf(o);
		if( i != -1)
		{
			remove(i);
			eliminado = true;
		}
		return eliminado;
	}

	/**
	 * Elimina el elemento de la posicin o de la lista.
	 * post: todos los elementos despus de pos se han corrido una posicin a la izquierda
	 * @return el elemento eliminado
	 * @throws IndexOutOfBoundsException si la posicin est fuera de la lista (pos < 0 || pos > size())
	 */
	public T remove(int pos) 
	{
		//TODO complete segn la documentacin
		
	}


	/**
	 * Elimina todos los elementos contenidos en la coleccin c que existan en la lista
	 * @param c la coleccin de elementos que se desea eliminar de la lista
	 * @return true en caso de que la lista se haya modificado o false en caso contrario
	 * @throws ClassCastException En caso de algn elemento t de c no sea de tipo T. En este caso se eliminan los elementos previos a t y 
	 * se lanza la excepcin una vez se llegue a t
	 * @throws NullPointerException En caso que algn elemento t de c sea nulo. En este caso se eliminan los elementos previos a t y 
	 * se lanza la excepcin una vez se llegue a t
	 */
	public boolean removeAll(Collection<?> c) {
		boolean modificado = false;
		Iterator<?> it = c.iterator();
		while(it.hasNext())
		{
			modificado |= remove(it.next());
		}
		return modificado;
	}


	/**
	 * Elimina todos los elementos NO contenidos en la coleccin c que existan en la lista
	 * @param c la coleccin de elementos que se desea mantener en la lista
	 * @return true en caso de que la lista se haya modificado o false en caso contrario
	 */
	public boolean retainAll(Collection<?> c) 
	{
		//TODO Complete segn la documentacin
		boolean modificado = false;
		NodoListaDoble<T> actual = primero;
		while(actual != null)
		{
			T elem = actual.darElemento();
			actual = actual.darSiguiente();
			if(!c.contains(elem))
			{
				modificado |= remove(elem);
			}
		}
		return modificado;
	}


	/**
	 * Reemplaza el elemento de la posicin pos por el elemento t
	 * @param pos la posicin que se desea reemplazar
	 * @param t el elemento con el que se reeemplazar el elemento de pos
	 * @return el elemento que se encontraba previamente en pos
	 * @throws NullPointerException Se lanza en caso que  t  sea nulo. 
	 * @throws IllegalArgumentException Se lanza en caso que ya exista el elemento t en la lista.
	 * @throws IndexOutOfBoundsException si la posicin est fuera de la lista (pos < 0 || pos > size())
	 */
	public T set(int pos, T t) 
	{
		//TODO complete segn la documentacin
		
	}

	/**
	 * Indica la cantidad de elementos en la lista
	 * @return nmero de elementos en la lista
	 */
	public int size() 
	{
		//TODO complete segn la documentacin
		// Recorra la lista contando cuantos elementos hay
		
	}


	/**
	 * Devuelve una sublista de nuevos elementos, que inicia en el elemento de la posicin desde (inclusivo) y termina en el elemento de la posicin hasta (exclusivo)
	 * @param desde Posicin desde la que se quiere construir la sublista (inclusivo)
	 * @param hasta Posicin hsata la que se quiere construir la sublista (exclusivo)
	 * @return Una ListaSencilla con los elementos de la posicin desde hasta la posicin hasta. En caso que desde >= hasta  se devuelve una lista vacia
	 * @throws IndexOutOfBoundsException si las posiciones desde y hasta estn fuera de la lista (pos < 0 || pos > size())
	 */
	public List<T> subList(int desde, int hasta) 
	{
		//TODO complte segn la documentacin
		
	}

	/**
	 * Devuelve un nuevo arreglo de objetos con los elementos T de la lista
	 * @return arreglo con los elementos de la lista
	 */
	public Object[] toArray() {
		
		//TODO complete segn la documentacin
		
	}

	/**
	 * Retorna un arreglo de T con los elementos T de la lista. Si los elementos de la lista caben en el parmetro se devuelve all
	 *  de lo contrario se devuelven en un nuevo arreglo. En caso que el parmetro tenga ms espacio que elementos en la lista el elemento
	 *  inmediatamente posterior al ltimo elemento de la lista en el arreglo debe ser null
	 *  @param array arreglo donde se guardaran los elementos de la lista
	 *  @return un arreglo con los elementos T de la lista
	 *  @throws NullPointerException Si el arreglo por parmetro es nulo
	 */
	public <T> T[] toArray(T[] array) throws NullPointerException
	{
		if( array == null )
		{
			throw new NullPointerException("El arreglo no puede ser nulo");
		}
		if( array.length < size())
		{
			return (T[])toArray();
		}
		else
		{
			NodoListaDoble actual = primero;
			int  i = 0;
			while(actual != null)
			{
				array[i] = (T)actual.darElemento();
				actual = actual.darSiguiente();
				i++;
			}
			if(array.length > size())
			{
				array[i] = null;
			}
			return array;
		}
		
		
	}

}
