package uniandes.cupi2.sitp.interfaz;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import uniandes.cupi2.sitp.mundo.Dia;
import uniandes.cupi2.sitp.mundo.FabricaRutas;
import uniandes.cupi2.sitp.mundo.IRuta;
import uniandes.cupi2.sitp.mundo.Parada;
import uniandes.cupi2.sitp.mundo.Troncal;

/**
 * Dialogo que permite agregar una nueva ruta de transmilenio
 * @author alvar-go
 *
 */
public class DialogoAgregarArticulado extends DialogoAgregarRuta {
	
	/**
	 * Constante de serializacin
	 */
	private static final long serialVersionUID = 1L;
	
	/**
	 * Posibles troncales por las que puede pasar la ruta
	 */
	private JCheckBox[] troncales;

	/**
	 * crea un nuevo dialogo para agregar rutas de transmilenio
	 * @param interfaz la clase principal de la aplicacin
	 * @param x coordenada en x de la primera parada del articulado
	 * @param y coordenada en y de la primera parada del articulado
	 */
	public DialogoAgregarArticulado(InterfazSITP interfaz, int x, int y) 
	{
		super(interfaz, x, y);
		
	}

	
	protected void inicializarCamposEspecificos() 
	{
		setSize(300, 500);
		setTitle("Agregar ruta transmilentio (Articulado)");
		
		troncales = new JCheckBox[Troncal.values().length];
		int k = 0;
		for (Troncal troncal : Troncal.values()) 
		{
			troncales[k] = new JCheckBox(troncal.toString());
			k++;
		}
		//TODO Agregue las troncales usando cal panelDatos usando el gbcPanelDatos
		//Ayuda: use como referencia el mtodo inicializar campos comunes de la clase DialogoAgregarRuta
		

	}

	
	protected void agregarRuta() 
	{
		String nombre = txtNombre.getText();
		String nombreParada = txtNombreParada.getText();
		String inicio = (String)horaInicio.getSelectedItem();
		String fin = (String)horaFin.getSelectedItem();
		
		if( nombre == null || nombre.trim().equals("") || nombreParada == null || nombreParada.trim().equals("") )
		{
			JOptionPane.showMessageDialog(this, "Debe completar todos los datos", "Error", JOptionPane.ERROR_MESSAGE);
		}
		else if( horaFin.getSelectedIndex() <= horaInicio.getSelectedIndex())
		{
			JOptionPane.showMessageDialog(this, "La hora de finalizacin debe ser posterior a la de inicio", "Error", JOptionPane.ERROR_MESSAGE);
		}
		else
		{
			Calendar c = Calendar.getInstance();
			String[] in = inicio.split(":");
			String[] fi = fin.split(":");
			c.set(Calendar.HOUR_OF_DAY, Integer.parseInt(in[0]));
			c.set(Calendar.MINUTE, Integer.parseInt(in[1]));
			Date din = c.getTime();
			
			c.set(Calendar.HOUR_OF_DAY, Integer.parseInt(fi[0]));
			c.set(Calendar.MINUTE, Integer.parseInt(fi[1]));
			Date dfin = c.getTime();
			
			Parada p = new Parada(nombreParada, xPrimeraParada, yPrimeraParada);
			List<Dia> ds = new ArrayList<Dia>();
			
			for (int i = 0; i < dias.length; i++) 
			{
				JCheckBox d = dias[i];
				if(d.isSelected())
				{
					ds.add(Dia.values()[i]);
				}
			}
			
			List<Troncal> ts = new ArrayList<Troncal>();
			
			for (int i = 0; i < troncales.length; i++) 
			{
				JCheckBox t = troncales[i];
				if(t.isSelected())
				{
					ts.add(Troncal.values()[i]);
				}
			}
			
			IRuta r = FabricaRutas.crearArticulado(nombre, ds, p, din, dfin, ts);
			principal.agregarRuta(r);
			
		}
	}
}
