package uniandes.cupi2.sitp.interfaz;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JOptionPane;

import uniandes.cupi2.sitp.mundo.Dia;
import uniandes.cupi2.sitp.mundo.IRuta;
import uniandes.cupi2.sitp.mundo.Parada;

/**
 * Dialogo abstracto que acopla el comportamiento de todos los dialogos que permite agregar rutas
 * @author alvar-go
 *
 */
public abstract class DialogoAgregarNoTransmilenio extends DialogoAgregarRuta {

	/**
	 * Constante de serializacin
	 */
	private static final long serialVersionUID = 1L;


	/**
	 * Crea un nuevo dialogo para agregar una ruta
	 * @param interfaz la clase principal de la aplicacin
	 * @param x coordenada en x de la primera parada del alimentador
	 * @param y coordenada en y de la primera parada del alimentador
	 */
	public DialogoAgregarNoTransmilenio(InterfazSITP interfaz, int x, int y) 
	{
		super(interfaz, x, y);
		
	}

	
	protected void inicializarCamposEspecificos() 
	{
		setSize(300, 500);

	}
	
	protected void agregarRuta() 
	{
		String nombre = txtNombre.getText();
		String nombreParada = txtNombreParada.getText();
		String inicio = (String)horaInicio.getSelectedItem();
		String fin = (String)horaFin.getSelectedItem();
		
		if( nombre == null || nombre.trim().equals("") || nombreParada == null || nombreParada.trim().equals("") )
		{
			JOptionPane.showMessageDialog(this, "Debe completar todos los datos", "Error", JOptionPane.ERROR_MESSAGE);
		}
		else if( horaFin.getSelectedIndex() <= horaInicio.getSelectedIndex())
		{
			JOptionPane.showMessageDialog(this, "La hora de finalizacin debe ser posterior a la de inicio", "Error", JOptionPane.ERROR_MESSAGE);
		}
		else
		{
			Calendar c = Calendar.getInstance();
			String[] in = inicio.split(":");
			String[] fi = fin.split(":");
			c.set(Calendar.HOUR_OF_DAY, Integer.parseInt(in[0]));
			c.set(Calendar.MINUTE, Integer.parseInt(in[1]));
			Date din = c.getTime();
			
			c.set(Calendar.HOUR_OF_DAY, Integer.parseInt(fi[0]));
			c.set(Calendar.MINUTE, Integer.parseInt(fi[1]));
			Date dfin = c.getTime();
			
			Parada p = new Parada(nombreParada, xPrimeraParada, yPrimeraParada);
			List<Dia> ds = new ArrayList<Dia>();
			
			for (int i = 0; i < dias.length; i++) 
			{
				JCheckBox d = dias[i];
				if(d.isSelected())
				{
					ds.add(Dia.values()[i]);
				}
			}
			
			IRuta r = crearRuta(nombre, ds, p, din, dfin);
			principal.agregarRuta(r);
			
		}
		

	}


	/**
	 * Crea una nueva ruta del tipo que se quiere agregar
	 * @param nombre el nombre de la ruta. nombre != null && nombre != ""
	 * @param ds los das en que opera la ruta. dias != null
	 * @param p la primera parada de la ruta. p != null
	 * @param din hora de inicio de operacin de la ruta. din != null
	 * @param dfin hora de fin de operacin de la ruta. dfin != null && dfin > din
	 * @return una nueva ruta del tipo que se quiere crear
	 */
	protected abstract IRuta crearRuta(String nombre, List<Dia> ds, Parada p, Date din, Date dfin);

}
