package uniandes.cupi2.sitp.interfaz;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import uniandes.cupi2.sitp.mundo.Dia;

public abstract class DialogoAgregarRuta extends JDialog implements ActionListener
{
	/**
	 * Constante de serializacin
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Cosntante para el botn de aceptar
	 */
	public final static String ACEPTAR= "Crear";
	
	/**
	 * Constante para el botn de cancelar
	 */
	public final static String CANCELAR= "Cancelar";
	
	/**
	 * Botn para crear la ruta
	 */
	private JButton btAceptar;
	
	/**
	 * Botn para cerrar el dialogo
	 */
	private JButton btCenelar;
	
	/**
	 * Caja de texto para el nombre de la ruta
	 */
	protected JTextField txtNombre;
	
	/**
	 * Caja de texto para el nombre de la primera parada de la ruta
	 */
	protected JTextField txtNombreParada;
	
	/**
	 * Combo para seleccionar la hora de inicio de operacin
	 */
	protected JComboBox horaInicio;
	
	/**
	 * Combo para la hora de fin de la operacin
	 */
	protected JComboBox horaFin;
	
	/**
	 * Panel donde se ingresan los datos de la ruta
	 */
	protected JPanel panelDatos;
	
	/**
	 * Layout usado para el panel de datos
	 */
	protected GridBagConstraints gbcPanelDatos;
	
	/**
	 * Posibles das en que puede operar la ruta
	 */
	protected JCheckBox[] dias;
	
	/**
	 * Clase principal de la aplicacin
	 */
	protected InterfazSITP principal;
	
	/**
	 * Coordenada en x de la primera parada de la ruta
	 */
	protected int xPrimeraParada;
	
	/**
	 * Coordenada en y de la primera parada de la ruta
	 */
	protected int yPrimeraParada;
	
	/**
	 * @param interfaz la clase principal de la aplicacin
	 * @param x coordenada en x de la primera parada del alimentador
	 * @param y coordenada en y de la primera parada del alimentador
	 */
	public DialogoAgregarRuta(InterfazSITP interfaz, int x, int y)
	{
		principal = interfaz;
		xPrimeraParada = x;
		yPrimeraParada = y;
		
		setLayout( new BorderLayout() );
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		
		
		inicializarCamposComunes();
		inicializarCamposEspecificos();
		pack();
		setLocationRelativeTo(null);
	}
	
	
	/**
	 * Inicializa los campos comunes a todas las rutas
	 */
	private  void inicializarCamposComunes(){
		txtNombre = new JTextField();
		txtNombre.setPreferredSize(new Dimension(150, 30));
		txtNombre.setMaximumSize(new Dimension(150, 30));
		txtNombre.setMinimumSize(new Dimension(150, 30));
		txtNombreParada = new JTextField();
		txtNombreParada.setPreferredSize(new Dimension(150, 30));
		txtNombreParada.setMaximumSize(new Dimension(150, 30));
		txtNombreParada.setMinimumSize(new Dimension(150, 30));
		horaInicio = new JComboBox();
		horaFin = new JComboBox();
		dias = new JCheckBox[7];
		int k = 0;
		for (Dia d : Dia.values()) {
			dias[k] = new JCheckBox(d.toString());
			k++;
		}

		
		for (int i = 4; i < 23; i++) 
		{
			String h1 = i + ":00";
			String h2 = i + ":30";
			
			horaInicio.addItem(h1);
			horaInicio.addItem(h2);
			
			horaFin.addItem(h1);
			horaFin.addItem(h2);
		}
		
		panelDatos = new JPanel(new GridBagLayout());
		gbcPanelDatos = new GridBagConstraints( );
		
		gbcPanelDatos.gridx = 0;
		gbcPanelDatos.gridy = 0;
		gbcPanelDatos.fill = GridBagConstraints.BOTH;
		panelDatos.add( new JLabel("Nombre ruta: "), gbcPanelDatos);
		
		gbcPanelDatos.gridx = 1;
		panelDatos.add(txtNombre, gbcPanelDatos);
		
		gbcPanelDatos.gridy = gbcPanelDatos.gridy + 1;
		gbcPanelDatos.gridx = 0;
		panelDatos.add(new JLabel("Nombre primera parada: "), gbcPanelDatos);
		
		gbcPanelDatos.gridx = 1;
		panelDatos.add(txtNombreParada, gbcPanelDatos);
		
		gbcPanelDatos.gridy = gbcPanelDatos.gridy + 1;
		gbcPanelDatos.gridx = 0;
		panelDatos.add(new JLabel("Hora de inicio: "), gbcPanelDatos);
		
		gbcPanelDatos.gridx = 1;
		panelDatos.add(horaInicio, gbcPanelDatos);
		
		
		gbcPanelDatos.gridy = gbcPanelDatos.gridy + 1;
		gbcPanelDatos.gridx = 0;
		panelDatos.add(new JLabel("Hora de fin: "), gbcPanelDatos);
		
		gbcPanelDatos.gridx = 1;
		panelDatos.add(horaFin, gbcPanelDatos);
		
		gbcPanelDatos.gridy = gbcPanelDatos.gridy + 1;
		gbcPanelDatos.gridx = 0;
		gbcPanelDatos.gridwidth = 2;
		panelDatos.add(new JLabel("Das de operacin: "), gbcPanelDatos);
		
		gbcPanelDatos.gridwidth = 1;
		
		for (int i = 0; i < dias.length; i++) 
		{
			gbcPanelDatos.gridy = gbcPanelDatos.gridy + (i%2==0?1:0);
			gbcPanelDatos.gridx = i%2;
			panelDatos.add(dias[i], gbcPanelDatos);
		}
		
		
		add(panelDatos, BorderLayout.CENTER);
		
		JPanel panelBotones = new JPanel(new GridLayout(1,2,5,5));
		btAceptar = new JButton(ACEPTAR);
		btAceptar.setActionCommand(ACEPTAR);
		btAceptar.addActionListener(this);
		panelBotones.add(btAceptar);
		
		btCenelar = new JButton(CANCELAR);
		btCenelar.setActionCommand(CANCELAR);
		btCenelar.addActionListener(this);
		panelBotones.add(btCenelar);
		
		add(panelBotones, BorderLayout.SOUTH);
	}

	public void actionPerformed(ActionEvent e) 
	{
		String c = e.getActionCommand();
		if(c.equals(ACEPTAR))
		{
			agregarRuta();
			super.dispose();
		}
		if(c.equals(CANCELAR))
		{
			super.dispose();
		}
		
	}

	/**
	 * Mtodo para agregar la ruta creada
	 */
	protected abstract void agregarRuta();
	
	/**
	 * Mtodo para inicializar los campos especficos de cada ruta
	 */
	protected abstract void inicializarCamposEspecificos();

}
