package uniandes.cupi2.sitp.interfaz;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;

/**
 * Dialogo que se presenta al ahcer doble clic en el mapa
 * @author alvar-go
 *
 */
public class DialogoOpciones extends JDialog implements ActionListener
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Constante para agregar una ruta de transmilentio
	 */
	public final static String AGREGAR_RUTA_T = "Agregar ruta Transmilenio";
	
	/**
	 * Constante para agregar una ruta alimentadora
	 */
	public final static String AGREGAR_RUTA_A = "Agregar ruta Alimentadora";
	
	/**
	 * Constante para agregar una ruta urbana
	 */
	public final static String AGREGAR_RUTA_U = "Agregar ruta Bus Urbano";
	
	/**
	 * Constante para agregar una parada a la ruta
	 */
	public final static String AGREGAR_PARADA_RUTA = "Agregar parada a ruta";
	
	/**
	 * Constante para buscar una ruta cercana al punto seleccionado
	 */
	public final static String BUSCAR_RUTA_CERCANA = "Buscar ruta cercana";
	
	/**
	 * Bton para agregar ruta de transmilenio
	 */
	private JButton btAgregarRutaT;
	
	/**
	 * Botn para agregar ruta alimentadora
	 */
	private JButton btAgregarRutaA;
	
	/**
	 * Botn para agregar ruta urbana
	 */
	private JButton btAgregarRutaU;
	
	/**
	 * Botn para agregar parada
	 */
	private JButton btAgregarParada;
	
	/**
	 * Botn para buscar ruta cercana
	 */
	private JButton btBuscarCercana;
	
	/**
	 * Clase principal de la aplicacin
	 */
	private InterfazSITP principal;
	
	/**
	 * Coordenada en x del punto seleccionado
	 */
	private int xSeleccionado;
	
	/**
	 * Coordenada en y del punto seleccionado
	 */
	private int ySeleccionado;
	
	/**
	 * Crea un dialogo de opciones
	 * @param interfaz clase principal de la aplicacin
	 * @param x la coordenada en x del punto seleccionado
	 * @param y la coordenada en y del punto seleccionado
	 */
	public DialogoOpciones(InterfazSITP interfaz, int x, int y)
	{
		principal = interfaz;
		xSeleccionado = x;
		ySeleccionado = y;
		
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setSize(200, 300);
		setTitle("Opciones");
		
		setLayout(new GridLayout( 5,1,5,5));
		
		btAgregarRutaT = new JButton(AGREGAR_RUTA_T);
		btAgregarRutaT.setActionCommand(AGREGAR_RUTA_T);
		btAgregarRutaT.addActionListener(this);
		add(btAgregarRutaT);
		
		btAgregarRutaA = new JButton(AGREGAR_RUTA_A);
		btAgregarRutaA.setActionCommand(AGREGAR_RUTA_A);
		btAgregarRutaA.addActionListener(this);
		add(btAgregarRutaA);
		
		btAgregarRutaU = new JButton(AGREGAR_RUTA_U);
		btAgregarRutaU.setActionCommand(AGREGAR_RUTA_U);
		btAgregarRutaU.addActionListener(this);
		add(btAgregarRutaU);
		
		btAgregarParada = new JButton(AGREGAR_PARADA_RUTA);
		btAgregarParada.setActionCommand(AGREGAR_PARADA_RUTA);
		btAgregarParada.addActionListener(this);
		add(btAgregarParada);
		
		btBuscarCercana = new JButton(BUSCAR_RUTA_CERCANA);
		btBuscarCercana.setActionCommand(BUSCAR_RUTA_CERCANA);
		btBuscarCercana.addActionListener(this);
		add(btBuscarCercana);
		
		setLocationRelativeTo(null);
		pack();
	}


	@Override
	public void actionPerformed(ActionEvent e) 
	{
		String c = e.getActionCommand();
		if(c.equals(AGREGAR_RUTA_T))
		{
			DialogoAgregarArticulado d = new DialogoAgregarArticulado(principal, xSeleccionado, ySeleccionado);
			super.dispose();
			d.setVisible(true);
		}
		if(c.equals(AGREGAR_RUTA_U))
		{
			//TODO muestre el dialogo para crear bus urbano. Siga los siguientes pasos
			//1. Cree el dialogo
			//2. Cierre el dialogo actual
			//3. Muestre el dialogo

		}
		if(c.equals(AGREGAR_RUTA_A))
		{
			DialogoAgregarAlimentador d = new DialogoAgregarAlimentador(principal, xSeleccionado, ySeleccionado);
			super.dispose();
			d.setVisible(true);
		}
		if(c.equals(AGREGAR_PARADA_RUTA))
		{
			super.dispose();
			principal.agregarParada(xSeleccionado, ySeleccionado);
			
			
		}
		if(c.equals(BUSCAR_RUTA_CERCANA))
		{
			super.dispose();
			principal.buscarRutaCercana(xSeleccionado, ySeleccionado);
		}
		
		
	}

}
