package uniandes.cupi2.sitp.interfaz;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import uniandes.cupi2.sitp.manejoArchivos.Serializador;
import uniandes.cupi2.sitp.mundo.Articulado;
import uniandes.cupi2.sitp.mundo.Dia;
import uniandes.cupi2.sitp.mundo.IRuta;
import uniandes.cupi2.sitp.mundo.Parada;
import uniandes.cupi2.sitp.mundo.SITP;
import uniandes.cupi2.sitp.mundo.Troncal;

/**
 * Clase principal de la aplicacin
 * @author alvar-go
 *
 */
public class InterfazSITP extends JFrame {

	/**
	 * Constante de la serializacin
	 */
	private static final long serialVersionUID = 1L;
	
	/**
	 * Ruta para guardar la informacin del programa
	 */
	public final static String RUTA = "data/sitp.data";

	/**
	 * Panel mapa de la ciudad
	 */
	private PanelMapa panelMapa;
	
	/**
	 * Panel de lista de rutas del sistema
	 */
	private PanelListaRutas panelListaRutas;
	
	/**
	 * Panel de lista de paradas de una ruta
	 */
	private PanelListaParadas panelListaParadas;
	
	/**
	 * Panel de extensin
	 */
	private PanelExtension panelExtension;
	
	/**
	 * Panel con los datos de una ruta
	 */
	private PanelDatosRuta panelDatosRuta;
	
	/**
	 * Clase principal del mundo
	 */
	private SITP sitp;
	
	/**
	 * Crea la ventana de la aplicacin
	 */
	public InterfazSITP()
	{
		Dimension d = getToolkit().getScreenSize();
		setSize(d.width,480);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setResizable(false);
		setTitle("SITP");
		
		setLayout(new BorderLayout());
		
		panelMapa = new PanelMapa(this);
		JScrollPane scroll = new JScrollPane(panelMapa);
		scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		add(scroll, BorderLayout.CENTER);
		
		JPanel panelListas = new JPanel(new GridLayout(1,2,5,5));
		Dimension d2 = new Dimension((int) (d.getWidth()*0.35),0);
		panelListas.setPreferredSize(d2);
		panelListas.setMaximumSize(d2);
		panelListas.setMinimumSize(d2);
		
		panelListaRutas = new PanelListaRutas(this);
		panelListas.add(panelListaRutas);
		
		panelListaParadas = new PanelListaParadas(this);
		panelListas.add(panelListaParadas);
		
		add(panelListas, BorderLayout.WEST);
		
		panelDatosRuta = new PanelDatosRuta();
		Dimension d3 = new Dimension((int) (d.getWidth()*0.2),0);
		panelDatosRuta.setPreferredSize(d3);
		panelDatosRuta.setMaximumSize(d3);
		panelDatosRuta.setMinimumSize(d3);
		add(panelDatosRuta, BorderLayout.EAST);
		
		panelExtension = new PanelExtension(this);
		add(panelExtension, BorderLayout.SOUTH);
		
		setLocationRelativeTo(null);
		
		sitp = new SITP();
		try 
		{
			sitp = Serializador.cargar(RUTA, sitp);
		} 
		catch (Exception e) 
		{
			JOptionPane.showMessageDialog(null, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}
		
		panelListaRutas.actualizar(sitp.darRutas());
		
		sitp.addObserver(panelMapa);

		sitp.cambiarRutaSeleccionada(panelListaRutas.darSeleccionado());
		
		
	}
	
	/**
	 * Actualiza la ruta seleccionada
	 * @param ruta ruta que se desea mostrar
	 */
	public void actualizarRuta(IRuta ruta) 
	{
		sitp.cambiarRutaSeleccionada(ruta);
		panelMapa.actualizar();
		panelListaParadas.actualizar(ruta.darParadas());
		panelDatosRuta.actualizar(ruta);
		
	}
	
	/**
	 * Cambia la parada seleccionada
	 * @param seleccionada la parada seleccionada
	 */
	public void actualizarParada(Parada seleccionada) 
	{
		panelMapa.cambiarParada(seleccionada);
		panelMapa.actualizar();
		
	}
	
	/**
	 * Agrega una nueva ruta al sistema
	 * @param r la ruta que se desea agregar
	 */
	public void agregarRuta(IRuta r) 
	{
		try 
		{
			sitp.agregarRuta(r);
			panelListaRutas.actualizar(sitp.darRutas());
			
		} 
		catch (Exception e) 
		{
			JOptionPane.showMessageDialog( this, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE );
		}
		
		
	}

	public void agregarParada(int xSeleccionado, int ySeleccionado) {
		String p = JOptionPane.showInputDialog(this, "Ingrese el nombre de la parada", "Nueva parada", JOptionPane.PLAIN_MESSAGE);
		IRuta r = (IRuta)panelListaRutas.darSeleccionado();
		if( r != null)
		{
			try 
			{
				r.agregarParada(new Parada(p, xSeleccionado, ySeleccionado));
				panelListaParadas.actualizar(r.darParadas());
				panelMapa.actualizar();
			} catch (Exception e) 
			{
				JOptionPane.showMessageDialog( this, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE );
			}
		}
		else
		{
			JOptionPane.showMessageDialog( this, "No hay ruta seleccionada", "Error", JOptionPane.ERROR_MESSAGE );
		}
		
	}

	/**
	 * Busca una ruta a patir de parte de su nombre
	 */
	public void buscarRuta()
	{
		String n = JOptionPane.showInputDialog(this, "Ingrese el nombre o parte del nombre de la ruta", "Buscar ruta", JOptionPane.PLAIN_MESSAGE);
		IRuta r = sitp.buscarRutaNombre(n);
		if( r == null )
		{
			JOptionPane.showMessageDialog( this, "No hay rutas con el nombre dado", "Busqueda", JOptionPane.WARNING_MESSAGE );
		}
		else
		{
			JOptionPane.showMessageDialog( this, "Se encontr la ruta: " + r.darIdentificador(), "Busqueda", JOptionPane.INFORMATION_MESSAGE );
			panelListaRutas.cambiarSeleccionado(r);
		}
		
	}

	/**
	 * Elimina la ruta seleccionada
	 */
	public void eliminarRuta() 
	{
		IRuta r = panelListaRutas.darSeleccionado();
		if( r == null )
		{
			 JOptionPane.showMessageDialog( this, "Seleccione una ruta", "Error", JOptionPane.ERROR_MESSAGE );
		}
		else
		{
			sitp.eliminarRuta(r);
			panelListaRutas.actualizar(sitp.darRutas());
			
		}
	}
	
	/**
	 * Limpia la informacin mostrada
	 */
	public void limpiar() 
	{
		sitp.cambiarRutaSeleccionada(null);
		panelDatosRuta.actualizar(null);
		panelListaParadas.actualizar(new ArrayList<Parada>());	
	}
	
	public void dispose()
	{
		try 
		{
			Serializador.guardar(RUTA, sitp);

	        JOptionPane.showMessageDialog( this, "Se gurad la informacin", "Cerrar", JOptionPane.INFORMATION_MESSAGE );
		} 
		catch (Exception e)
		{
			 JOptionPane.showMessageDialog( this, e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE );
		}
		super.dispose();
	}
	
	 /**
     * Mtodo para la extensin 1.
     */
    public void reqFuncOpcion1( )
    {
        String resultado = sitp.metodo1( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    /**
     * Mtodo para la extensin 2.
     */
    public void reqFuncOpcion2( )
    {
        String resultado = sitp.metodo2( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }
	
	public static void main(String[] args) {
		InterfazSITP i = new InterfazSITP();
		i.setVisible(true);
	}

	/**
	 * Busca una ruta cercana al punto dado por parmetro
	 * @param xSeleccionado coordenada en X del punto
	 * @param ySeleccionado coordenada en Y del punto
	 */
	public void buscarRutaCercana(int xSeleccionado, int ySeleccionado) 
	{
		IRuta r = sitp.darRutaCercana(xSeleccionado, ySeleccionado);
		if( r!= null)
		{
			panelListaRutas.cambiarSeleccionado(r);
			JOptionPane.showMessageDialog( this, "Se encontr la ruta " + r.darNombre(), "Cercanas", JOptionPane.INFORMATION_MESSAGE );
		
		}
		else 
		{
			JOptionPane.showMessageDialog( this, "No hay rutas cercanas al punto", "Cercanas", JOptionPane.INFORMATION_MESSAGE );
		}
		
	}

}
