package uniandes.cupi2.sitp.interfaz;

import java.awt.GridLayout;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import uniandes.cupi2.sitp.mundo.Articulado;
import uniandes.cupi2.sitp.mundo.IRuta;

/**
 * Panel para presentar los datos de una ruta
 * @author alvar-go
 *
 */
public class PanelDatosRuta extends JPanel
{

	/**
	 * Constante de serializacin
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Caja de texto del nombre de la ruta
	 */
	private JTextField txtNombre;
	
	/**
	 * Caja de texto con los das en que opera la ruta
	 */
	private JTextField txtDias;
	
	/**
	 * Caja de texto con el horario en que opera la ruta
	 */
	private JTextField txtHorario;
	
	/**
	 * Caja de texto con las troncales por las que opera la ruta
	 */
	private JTextField txtTroncales;
	
	/**
	 * Crea un nuevo panel para mostrar datos de ruta
	 */
	public PanelDatosRuta()
	{
		setBorder(BorderFactory.createTitledBorder("Informacin ruta"));
		setLayout(new GridLayout(5, 2, 5, 60));
		
		txtNombre = new JTextField();
		txtDias = new JTextField();
		txtHorario = new JTextField();
		txtTroncales = new JTextField();
		
		txtNombre.setEditable(false);
		txtDias.setEditable(false);
		txtHorario.setEditable(false);
		txtTroncales.setEditable(false);
		
		add(new JLabel("Nombre: "));
		add(txtNombre);
		
		add(new JLabel("Dis de operacin: "));
		add(txtDias);
		
		add(new JLabel("Horario: "));
		add(txtHorario);
		
		add(new JLabel("Tronclaes:  "));
		add(txtTroncales);
	}
	
	/**
	 * Actualiza el panel mostrando la ruta que llega por parmetro
	 * @param r la ruta que se desea mostrar
	 */
	public void actualizar(IRuta r)
	{
		if(r != null)
		{
			txtNombre.setText(r.darNombre());
			txtDias.setText(r.darDiasFuncionamiento().toString());
			
			DateFormat df = new SimpleDateFormat("HH:mm");
			txtHorario.setText(df.format(r.darHoraInicioFuncionamiento()) + " - " + df.format(r.darHoraFinFuncionamiento()));
			
			if( r instanceof Articulado)
			{
				txtTroncales.setText(((Articulado)r).darTroncales().toString());
			}
			else
			{
				txtTroncales.setText("NA");
			}
		}
		else
		{
			txtNombre.setText("");
			txtDias.setText("");
			txtTroncales.setText("");
			txtHorario.setText("");
		}
	}
}
