package uniandes.cupi2.sitp.interfaz;

import java.awt.BorderLayout;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import uniandes.cupi2.sitp.mundo.Parada;

/**
 * Panel para mostrar la lista de paradas de una ruta
 * @author alvar-go
 *
 */
public class PanelListaParadas extends JPanel implements ListSelectionListener
{

	/**
	 * Constante de serializacin
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Lista de paradas
	 */
	private JList listaParadas;
	
	/**
	 * Clase principal de la aplicacin
	 */
	private InterfazSITP principal;
	
	/**
	 * Crea un nuevo panel de lista de paradas
	 * @param interfaz la clase principal de la aplicacin
	 */
	public PanelListaParadas(InterfazSITP interfaz)
	{
		principal = interfaz;
		setBorder(BorderFactory.createTitledBorder("Paradas"));
		setLayout( new BorderLayout() );
		
		listaParadas = new JList();
		listaParadas.addListSelectionListener(this);
		JScrollPane scroll = new JScrollPane(listaParadas);
		scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		add(scroll, BorderLayout.CENTER);
	}
	
	/**
	 * Actualiza la lista de paradas con la lista por parmetro
	 * @param paradas las paradas a mostrar
	 */
	public void actualizar(List<Parada> paradas)
	{
		listaParadas.removeAll();
		listaParadas.setListData(paradas.toArray());
		if(paradas.size() > 0)
		{
			listaParadas.setSelectedIndex(0);
		}
	}
	
	/**
	 * Devuelve la parada seleccionada
	 * @return parada seleccionada
	 */
	public Parada darSeleccionado()
	{
		return (Parada)listaParadas.getSelectedValue();
	}


	public void valueChanged(ListSelectionEvent e) 
	{
		Parada seleccionada = (Parada)listaParadas.getSelectedValue();
		principal.actualizarParada(seleccionada);
		
	}
}
