package uniandes.cupi2.sitp.interfaz;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import uniandes.cupi2.sitp.mundo.IRuta;

/**
 * Panel para mostrar la lista de rutas del sistema
 * @author alvar-go
 *
 */
public class PanelListaRutas extends JPanel implements ListSelectionListener, ActionListener
{
	/**
	 * Consante de serializacin
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Constante para buscar una ruta
	 */
	public final static String BUSCAR_RUTA = "Buscar ruta";
	
	/**
	 * Constante para eliminar una ruta
	 */
	public final static String ELIMINAR_RUTA = "Eliminar ruta";
	
	/**
	 * Lista de rutas
	 */
	private JList listaRutas;
	
	/**
	 * Botn para buscar ruta
	 */
	private JButton btBuscar;
	
	/**
	 * Botn para eliminar ruta
	 */
	private JButton btEliminar;
	
	/**
	 * Clase principal de la aplicacin
	 */
	private InterfazSITP principal;
	
	/**
	 * Construye un panel para mostrar rutas
	 * @param interfaz clase principal de la aplicacin
	 */
	public PanelListaRutas(InterfazSITP interfaz)
	{
		principal = interfaz;
		setBorder(BorderFactory.createTitledBorder("Rutas"));
		setLayout( new BorderLayout() );
		
		listaRutas = new JList();
		listaRutas.setCellRenderer(new RutaLista());
		listaRutas.addListSelectionListener(this);
		JScrollPane scroll = new JScrollPane(listaRutas);
		scroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scroll.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		add(scroll, BorderLayout.CENTER);
		
		JPanel panelBotones = new JPanel(new GridLayout(2,1,5,5));
		btBuscar = new JButton(BUSCAR_RUTA);
		btBuscar.setActionCommand(BUSCAR_RUTA);
		btBuscar.addActionListener(this);
		panelBotones.add(btBuscar);
		
		btEliminar = new JButton(ELIMINAR_RUTA);
		btEliminar.setActionCommand(ELIMINAR_RUTA);
		btEliminar.addActionListener(this);
		panelBotones.add(btEliminar);
		
		add(panelBotones, BorderLayout.SOUTH);
	}
	
	/**
	 * Actualiza la lista de rutas
	 * @param rutas rutas que se desean mostrar
	 */
	public void actualizar(List<IRuta> rutas)
	{
		listaRutas.removeAll();
		listaRutas.setListData(rutas.toArray());
		if( rutas.size() > 0)
		{
			
			listaRutas.setSelectedIndex(0);
		}
	}
	
	/**
	 * Devuelve la ruta seleccionada
	 * @return elemento seleccionado en la lista
	 */
	public IRuta darSeleccionado()
	{
		return (IRuta)listaRutas.getSelectedValue();
	}


	public void valueChanged(ListSelectionEvent e) 
	{
		IRuta ruta = (IRuta)listaRutas.getSelectedValue();
		if( ruta != null)
		{
			principal.actualizarRuta(ruta);
		}
		else 
		{
			principal.limpiar();
		}
		
	}


	public void actionPerformed(ActionEvent e) 
	{
		String c = e.getActionCommand();
		if( c.equals(BUSCAR_RUTA))
		{
			principal.buscarRuta();
		}
		if(c.equals(ELIMINAR_RUTA))
		{
			principal.eliminarRuta();
		}
		
	}

	public void cambiarSeleccionado(IRuta r) 
	{
		listaRutas.setSelectedValue(r, true);
		
	}
	
	
}
