package uniandes.cupi2.sitp.interfaz;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.imageio.ImageIO;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import uniandes.cupi2.sitp.mundo.IRuta;
import uniandes.cupi2.sitp.mundo.Parada;

/**
 * Panel para pintar el mapa
 * @author alvar-go
 *
 */
public class PanelMapa extends JPanel implements MouseListener, Observer
{
	
	/**
	 * Constante de serialziacin
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Imagen del mapa
	 */
	private BufferedImage mapa;
	
	/**
	 * Clase principal de la aplicacin
	 */
	private InterfazSITP principal;
	
	/**
	 * Coordenada en x del punto seleccionado
	 */
	private int xSeleccionado;
	
	/**
	 * Coordenada en y del punto seleccionado
	 */
	private int ySeleccionado;
	
	/**
	 * Ruta mostradas
	 */
	private IRuta ruta;
	
	/**
	 * Parada seleccionada
	 */
	private Parada parada;
	
	/**
	 * Crea un nuevo panel de mapa
	 * @param interfaz clase principal de la aplicacin
	 */
	public PanelMapa(InterfazSITP interfaz) 
	{
		principal = interfaz;
		setDoubleBuffered( true );
        setBorder( new TitledBorder( "" ) );
        addMouseListener(this);
        
        xSeleccionado = -1;
        ySeleccionado = -1;
        ruta = null;
        parada = null;
        try {
			
        	mapa = ImageIO.read(new File("data/imagenes/mapabogota2.png"));
        	setSize(mapa.getWidth(), mapa.getHeight());
        	setMinimumSize(new Dimension(mapa.getWidth(), mapa.getHeight()));
        	setMaximumSize(new Dimension(mapa.getWidth(), mapa.getHeight()));
        	setPreferredSize(new Dimension(mapa.getWidth(), mapa.getHeight()));
		} catch (Exception e) {
			
		}
	}
	
	/**
	 * Actualiza el panel
	 */
	public void actualizar()
	{
		repaint();
	}
	
	 /**
     * Mtodo llamado por la mquina virtual cuando hay que repintar el panel.
     * @param g Superficie del panel.
     */
    public void paintComponent( Graphics g )
    {
        super.paintComponent( g );

        update( ( Graphics2D )g );
    }

    /**
     * Actualiza la visualizacin del panel.
     * @param g Superficie del panel.
     */
    public void update( Graphics2D g )
    {
    	 if(mapa != null)
    	 {
    		 g.drawImage(mapa, 0, 0, null);
    	 }
    	 g.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );

    	 if(xSeleccionado != -1 && ySeleccionado != -1)
    	 {
    		 g.setColor(Color.GREEN);
    		 g.fillOval(xSeleccionado, ySeleccionado, 5, 5);
    		 xSeleccionado = -1;
    		 ySeleccionado = -1;
    	 }
    	 if(ruta != null)
    	 {
    		 g.setColor(ruta.darColor());
    		 List<Parada> paradas = ruta.darParadas();
    		 for (Parada p : ruta.darParadas()) 
    		 {
    			 g.fillOval(p.darX(), p.darY(), 5, 5);
    		 }
    		 for (int i = 0; i < paradas.size()-1; i++) 
    		 {
    			 Parada p1 = paradas.get(i);
    			 Parada p2 = paradas.get(i+1);
				g.drawLine(p1.darX(), p1.darY(), p2.darX(), p2.darY());
			 }
    		 if(parada != null)
    		 {
    			 g.setColor(Color.GREEN);
    			 g.fillOval(parada.darX(), parada.darY(), 5, 5);
    		 }
    	 }

    }
    
    /**
     * Cambia la aprada seleccionada
     * @param p la nueva parada seleccionada
     */
    public void cambiarParada(Parada p)
    {
    	parada = p;
    }


	public void mouseClicked(MouseEvent e) {
		if( e.getClickCount() == 2 )
		{
			xSeleccionado = e.getX();
			ySeleccionado = e.getY();
			actualizar();
			
			DialogoOpciones d = new DialogoOpciones(principal, xSeleccionado, ySeleccionado);
			d.setVisible(true);
		}
		
	}

	public void mouseEntered(MouseEvent e) {
		//No se implementa
		
	}


	public void mouseExited(MouseEvent e) {
		//No se implementa
		
	}


	public void mousePressed(MouseEvent e) {
		//No se implementa
		
	}


	public void mouseReleased(MouseEvent e) {
		//No se implementa
		
	}


	public void update(Observable observado, Object msj) 
	{
		if( msj == null )
		{
			ruta = null;
		}
		else if( msj instanceof IRuta)
		{
			ruta = (IRuta)msj;
		}
		actualizar();
		
	}
	

}
