package uniandes.cupi2.sitp.mundo;

import java.util.Date;
import java.util.List;

/**
 * Clase encargada de crear diferentes tipos de rutas
 * @author alvar-go
 *
 */
public class FabricaRutas 
{

	/**
	 * Crea una nueva ruta alimentadora con la informacin por parmetro
	 * @param nNombre el nombre de la ruta. nNombre != null && nNombre != ""
	 * @param nDias los dias en que opera la ruta. dias != null
	 * @param primeraParada la primera parada de la ruta. primeraParada != null
	 * @param nInicio la hora de inicio de la ruta. nInicio != null
	 * @param nFin la hora de finalizacin de la ruta. nFin != null && nFin > nInicio
	 * @return una nueva ruta alimentadora o null si se presenta algn problema
	 */
	public static IRuta crearAlimentador(String nNombre, List<Dia> nDias,Parada primeraParada, Date nInicio, Date nFin)
	{
		try 
		{
			Alimentador a = (Alimentador)Class.forName("uniandes.cupi2.sitp.mundo.Alimentador").newInstance();
			a.cambiarDias(nDias);
			a.cambiarNombre(nNombre);
			a.cambiarHoraFin(nFin);
			a.cambiarHoraInicio(nInicio);
			a.agregarParada(primeraParada);
			return a;
		} 
		catch (Exception e) 
		{
			return null;
		} 
		
	}
	
	/**
	 * Crea una nueva ruta de transmilenio con la informacin que llega por parmetro
	 * @param nNombre el nombre de la ruta. nNombre != null && nNombre != ""
	 * @param nDias los dias en que opera la ruta. dias != null
	 * @param primeraParada la primera parada de la ruta. primeraParada != null
	 * @param nInicio la hora de inicio de la ruta. nInicio != null
	 * @param nFin la hora de finalizacin de la ruta. nFin != null && nFin > nInicio
	 * @param troncales las troncales por las que pasa la ruta. troncales != null
	 * @return una nueva ruta de transmlinio o null si se presenta algn problema
	 */
	public static IRuta crearArticulado(String nNombre, List<Dia> nDias,Parada primeraParada, Date nInicio, Date nFin, List<Troncal> troncales)
	{
		try 
		{
			Articulado a = (Articulado)Class.forName("uniandes.cupi2.sitp.mundo.Articulado").getConstructor(List.class).newInstance(troncales);
			a.cambiarDias(nDias);
			a.cambiarNombre(nNombre);
			a.cambiarHoraFin(nFin);
			a.cambiarHoraInicio(nInicio);
			a.agregarParada(primeraParada);
			return a;
		} 
		catch (Exception e) 
		{
			return null;
		} 
	}
	
	/**
	 * Crea una nueva ruta urbana con la informacin que llega por parmetro
	 * @param nNombre el nombre de la ruta. nNombre != null && nNombre != ""
	 * @param nDias los dias en que opera la ruta. dias != null
	 * @param primeraParada la primera parada de la ruta. primeraParada != null
	 * @param nInicio la hora de inicio de la ruta. nInicio != null
	 * @param nFin la hora de finalizacin de la ruta. nFin != null && nFin > nInicio
	 * @return una nueva ruta de bus urnano o null si se presenta algn error
	 */
	public static IRuta crearBusUrbano(String nNombre, List<Dia> nDias,Parada primeraParada, Date nInicio, Date nFin)
	{
		//TODO complete el mtodo segn la documentacin. Use introspeccin. Use como referencia los otros mtodos de est classe

		
	}
	
}
