package uniandes.cupi2.sitp.mundo;

import java.awt.Color;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

import estructuras.listas.IUnicamenteIdentificado;

/**
 * Contrato que deben cumplir todas las rutas del sistema
 * @author alvar-go
 *
 */
public interface IRuta extends Serializable, IUnicamenteIdentificado
{
	/**
	 * Devuelve la lista de paradas de la ruta
	 * @return lista de paradas
	 */
	public List<Parada> darParadas();
	
	/**
	 * Agrega una nueva parada a la ruta
	 * @param parada la parada que se desea agregar. parada != null
	 * @throws Exception Se lanza en caso que ya exista la parada en la ruta
	 */
	public void agregarParada(Parada parada) throws Exception;
	
	/**
	 * Indica si la ruta pasa cerca a un punto dado
	 * @param x la coordenada en x del punto. x > 0
	 * @param y la coordenada en y del punto. y > 0
	 * @return true en caso que la ruta pase cerca al punto o false en caso contrario
	 */
	public boolean pasaCercaA(int x, int y);
	
	/**
	 * Devuelve el nombre de la ruta
	 * @return nombre de la ruta
	 */
	public String darNombre();
	
	/**
	 * Devuelve el color de la ruta
	 * @return color que identifica a la ruta
	 */
	public Color darColor();
	
	/**
	 * Devuelve la hora en que inica a operar la ruta
	 * @return hora de inicio de operacin
	 */
	public Date darHoraInicioFuncionamiento();
	
	/**
	 * Devuelve la hora en que termina de operar la ruta
	 * @return hora de fin de operacin
	 */
	public Date darHoraFinFuncionamiento();
	
	/**
	 * Devuelve una lista con los das en que funciona la ruta
	 * @return lista de das en que opera la ruta
	 */
	public List<Dia> darDiasFuncionamiento();
}
