
package uniandes.cupi2.sitp.mundo;

import java.io.Serializable;

import estructuras.listas.IUnicamenteIdentificado;

/**
 * Clase que representa la parada de una ruta
 * @author alvar-go
 *
 */
public class Parada implements IUnicamenteIdentificado, Serializable
{


    /**
     * Constante de serializacin de la clase.
     */
    private static final long serialVersionUID = 1L;


    /**
     * Nombre de la parada.
     */
    private String nombre;

    /**
     * Coordenada en x de la parada
     */
    private int x;
    
    /**
     * Coordenada en y de la parada
     */
    private int y;

    /**
     * Crea una nueva parada
     * @param pNombre el nombre de la parada
     * @param nX la coordenada en x de la parada. nX > 0
     * @param nY la coordenada en y de la parada. nY > 0
     */
    public Parada( String pNombre, int nX, int nY)
    {
        nombre = pNombre;
        x = nX;
        y = nY;
    }

    /**
     * Retorna el nombre de la parada.
     * @return Nombre de la parada.
     */
    public String darNombre( )
    {
        return nombre;
    }

    /**
     * Devuelve la coordenada en x de la parada
     * @return x
     */
    public int darX()
    {
    	return x;
    }
    
    /**
     * Devuelve la coordenada en y de la parada
     * @return y
     */
    public int darY()
    {
    	return y;
    }
    
    /**
     * Indica si la parada est cerca de un punto dado.
     * Esto es a menos de 30 unidades del punto pX, PY
     * @param pX la coordenada en x del punto que se quiere evaluar. pX>0
     * @param pY la coordenada en y del punto que se quiere evaluar. pY>0
     * @return true si la parada est cerca del punto o false en caso contrario
     */
    public boolean estaCerca(int pX, int pY)
    {
    	return Math.abs(x-pX) <= 30 && Math.abs(y-pY) <= 30;
    }

	public String darIdentificador() 
	{
		return nombre;
	}
	
	public String toString()
	{
		return nombre;
	}
}
