package uniandes.cupi2.sitp.mundo;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import estructuras.listas.IUnicamenteIdentificado;

/**
 * Clase abstracta que acopla el comportamiento comn de todas las rutas
 * @author alvar-go
 *
 */
public abstract class Ruta implements IRuta, IUnicamenteIdentificado
{

	/**
	 * Constante de serializacin
	 */
	private static final long serialVersionUID = 10L;
	
	/**
	 * Lista de paradas de la ruta
	 */
	protected List<Parada> paradas;
	
	/**
	 * Color que identifica a la ruta
	 */
	protected Color color;
	
	/**
	 * Hora de inicio de operacin de la ruta
	 */
	protected Date horaInicio;
	
	/**
	 * Hora de fin de operacin de la ruta
	 */
	protected Date horaFin;
	
	/**
	 * Lista de das en que opera la ruta
	 */
	protected List<Dia> dias;
	
	/**
	 * Nombre que identifica a la ruta
	 */
	protected String nombre;
	
	/**
	 * Crea una ruta sin valores
	 */
	public Ruta()
	{
		//TODO inicialice las paradas como una lista doblemente encadenada

	}
	
	
	public List<Parada> darParadas()
	{
		return paradas;
	}
	
	public void agregarParada(Parada p) throws Exception
	{
		paradas.add( p );
	}
	
	public boolean pasaCercaA(int x, int y)
	{
		for( int i = 0; i < paradas.size(); i++)
		{
			if(paradas.get(i).estaCerca(x, y))
			{
				return true;
			}
		}
		return false;
	}
	
	public String darNombre()
	{
		return nombre;
	}
	
	public Color darColor()
	{
		return color;
	}
	
	public Date darHoraInicioFuncionamiento()
	{
		return horaInicio;
	}
	
	public Date darHoraFinFuncionamiento()
	{
		return horaFin;
	}
	
	public List<Dia> darDiasFuncionamiento()
	{
		return dias;
	}
	
	public String darIdentificador() 
	{
		return nombre;
	}

	public void cambiarParadas(List<Parada> paradas) {
		this.paradas = paradas;
	}

	public void cambiarColor(Color color) {
		this.color = color;
	}

	public void cambiarHoraInicio(Date horaInicio) {
		this.horaInicio = horaInicio;
	}

	public void cambiarHoraFin(Date horaFin) {
		this.horaFin = horaFin;
	}

	public void cambiarDias(List<Dia> dias) {
		this.dias = dias;
	}

	public void cambiarNombre(String nombre) {
		this.nombre = nombre;
	}
	
	

}
