package uniandes.cupi2.sitp.mundo;

import java.io.Serializable;
import java.util.List;
import java.util.Observable;

import estructuras.listas.ListaSencilla;

/**
 * Clase que representa el sistema integrado de transporte
 * @author alvar-go
 *
 */
public class SITP extends Observable implements Serializable{

	/**
	 * Constante de serializacin
	 */
	private static final long serialVersionUID = 50L;

	/**
	 * Lista de rutas del sistema
	 */
	private List<IRuta> rutas;
	
	/**
	 * Ruta seleccionada actualmente
	 */
	private IRuta rutaSeleccionada;
	
	/**
	 * Crea un nuevo sistema integrado de transporte
	 * post: 	la lista de rutas se ha inicializado como vacia
	 * 			la ruta seleccionada se ha inicializado en null
	 */
	public SITP()
	{
		//TODO inicialice las rutas como una lista sencillamente encadenada

		rutaSeleccionada = null;
	}
	
	/**
	 * Agrega una nueva ruta al sistema
	 * @param r la ruta que se desea agregar. r != null
	 * @throws Exception Se lanza en caso que la ruta ya exista en el sistema
	 */
	public void agregarRuta(IRuta r) throws Exception
	{
		rutas.add(r);
	}


    /**
     * Devuelve la lista de rutas del sistema
     * @return rutas
     */
	public List<IRuta> darRutas() 
	{
		return rutas;
	}

	/**
	 * Cambia la ruta seleccionada por la ruta que llega por parmetro
	 * @param ruta nueva ruta seleccionada.
	 */
	public void cambiarRutaSeleccionada(IRuta ruta)
	{
		rutaSeleccionada = ruta;
		setChanged();
		notifyObservers(rutaSeleccionada);
		
	}

	/**
	 * Busca una ruta que contenga en su nombre el texto por parmetro
	 * @param parteId la parte del nombre de la ruta. parteId != null
	 * @return la primera ruta que en su nombre contenga el texto buscado o null en caso de que ninguna lo contenga
	 */
	public IRuta buscarRutaNombre(String parteId) 
	{
		IRuta buscada = null;
		for (int i = 0; i < rutas.size() && buscada == null; i++) 
		{
			IRuta r = rutas.get(i);
			if(r.darIdentificador().contains(parteId))
			{
				buscada = r;
			}
		}
		cambiarRutaSeleccionada(buscada);
		return buscada;
	}
	
	/**
	 * Elimina una ruta que llega por parmetro
	 * @param r la ruta que se desea eliminar. La ruta est en el sistema
	 */
	public void eliminarRuta(IRuta r)
	{
		rutas.remove(r);
	}
	
	/**
	 * Devuelve una ruta cercana al punto de coordenadas que llegan por parmetro
	 * @param x la coordenada en x del punto
	 * @param y la coordenada en y del punto
	 * @return la primera ruta cercana al punto o null en caso que ninguna ruta sea cercana al punto dado
	 */
	public IRuta darRutaCercana(int x, int y)
	{
		for (int i = 0; i < rutas.size(); i++) 
		{
			IRuta actual = rutas.get(i);
			if( actual.pasaCercaA(x, y))
			{
				cambiarRutaSeleccionada(actual);
				return actual;
			}
		}
		return null;
	}
	

    /**
     * Mtodo para la extensin1.
     * @return repuesta1.
     */
    public String metodo1( )
    {
        return "Respuesta 1.";
    }

    /**
     * Mtodo para la extensin2.
     * @return respuesta2.
     */
    public String metodo2( )
    {
        return "Respuesta 2.";
    }
	
	
}
